@@ -1,5 +1,6 @@
 import time
 import os
+from tqdm import tqdm
 
 
 def time_it(func):
@@ -26,7 +27,10 @@ def load_dir(input_path, level):
     elif level == 2:
         idx = 1
         for class_dir in os.listdir(input_path):
-            for img_name in enumerate(os.listdir(f"{input_path}/{class_dir}")):
+            if not os.path.isdir(f'{input_path}/{class_dir}'):
+                continue
+                
+            for img_name in os.listdir(f"{input_path}/{class_dir}"):
                 img_path = f"{input_path}/{class_dir}/{img_name}"
                 content = {"idx": idx, "class": class_dir,
                            'img_name': img_name+' ', 'img': img_path}
@@ -36,13 +40,16 @@ def load_dir(input_path, level):
     return content_table
 
 
-def handle_src(src, output_dir):
-    src = os.path.relpath(src, output_dir)
-    assert '..' not in src, 'The html file must in one of the parent folder of all images.'
+def handle_src(src, output_dir, rel_path=True):
+    if rel_path:
+        src = os.path.relpath(src, output_dir)
+        assert '..' not in src, 'The html file must in one of the parent folder of all images.'
+    else:
+        return src
     return src
 
 
-def generate_html_table(content_table, image_width='auto', image_height='auto', output_path=''):
+def generate_html_table(content_table, image_width='auto', image_height='auto', output_path='', float_precision=3, max_str_len=30, rel_path=True, save=True):
     """Generate html table
 
     Args:
@@ -50,6 +57,9 @@ def generate_html_table(content_table, image_width='auto', image_height='auto',
         width: image width
         height: image height
         output_path: output html path.
+        float_precision: Max precision of float values.
+        max_str_len: Max string length.
+        rel_path: Whether to use the relative path of input image and output path.
     """
     output_dir = os.path.split(output_path)[0]
     html = '<html>'
@@ -110,14 +120,14 @@ def generate_html_table(content_table, image_width='auto', image_height='auto',
     width = image_width
     height = image_height
     all_content_dict = []
-    for content_row in content_table:
+    for content_row in tqdm(content_table, desc='Generating rows...'):
         content_dict = {}
         for i, head in enumerate(heads):
-            content = content_row[head]
+            content = 'None' if not head in content_row else content_row[head]
             subhtml = ''
 
             if type(content) == dict:  # 图片，支持更丰富的样式
-                src = handle_src(content['src'], output_dir)
+                src = handle_src(content['src'], output_dir, rel_path)
                 alt = '' if not "alt" in content else content['alt']
                 title = '' if not "title" in content else content['title']
                 item_width = width if not "width" in content else content['width']
@@ -130,16 +140,22 @@ def generate_html_table(content_table, image_width='auto', image_height='auto',
 
             # 图片
             elif type(content) == str and os.path.splitext(content)[-1].lower() in ['.jpg', '.png', '.jpeg', '.gif']:
-                src = handle_src(content, output_dir)
+                src = handle_src(content, output_dir, rel_path)
                 subhtml += f"<img src={src} alt=\"{src}\" height={height} width={width}>"
 
             # 视频
             elif type(content) == str and os.path.splitext(content)[-1].lower() in ['.mp4', '.webm']:
-                src = handle_src(content, output_dir)
+                src = handle_src(content, output_dir, rel_path)
                 subhtml += f"<video src={src} alt=\"{src}\" height={height} width={width}>"
 
+            elif type(content) == float:
+                subhtml = f"{content:.{float_precision}f}"
+
             else:
-                subhtml = f"{content}"
+                if max_str_len > 0:
+                    subhtml = f"{str(content)[:max_str_len]}"
+                else:
+                    subhtml = str(content)
 
             content_dict[head] = subhtml
 
@@ -155,7 +171,9 @@ def generate_html_table(content_table, image_width='auto', image_height='auto',
     </script>
     """ % str(all_content_dict)
     html += '</body></html>'
-    if output_path != '':
+    if output_path == '':
+        output_path = './index.html'
+    if save == True:
         open(output_path, 'w').write(html)
 
     return html
